@@ -23,9 +23,29 @@ validateData <- function(d) {
 
 #' @noRd
 detectNCategories <- function(ipar) {
-  nd <- detectDimensions(ipar)
-  n_cats <- apply(ipar, 1, function(x) sum(!is.na(x)) - nd + 1)
-  return(n_cats)
+
+  p_type <- detectParameterization(ipar)
+
+  if (p_type == "ab") {
+    b_columns <- unique(c(
+      grep("^b$"     , names(ipar), value = TRUE),
+      grep("^b[1-9]$", names(ipar), value = TRUE),
+      grep("^cb$"     , names(ipar), value = TRUE),
+      grep("^cb[1-9]$", names(ipar), value = TRUE)
+    ))
+    n_cats <- apply(ipar, 1, function(x) length(na.omit(x[b_columns])) + 1)
+    return(n_cats)
+  }
+
+  if (p_type == "ad") {
+    d_columns <- unique(c(
+      grep("^d$"     , names(ipar), value = TRUE),
+      grep("^d[1-9]$", names(ipar), value = TRUE)
+    ))
+    n_cats <- apply(ipar, 1, function(x) length(na.omit(x[d_columns])) + 1)
+    return(n_cats)
+  }
+
 }
 
 #' (internal) construct a model
@@ -111,54 +131,68 @@ makeCalibrationModel <- function(d, dimensions, bound_cov) {
 #' @noRd
 makeParameterLayout <- function(d, dimensions, bound_cov) {
 
-  resp_data <- getResponse(d)
-  m         <- makeCalibrationModel(d, dimensions, bound_cov)
-  layout    <- mirt(resp_data, m, itemtype = "graded", pars = "values")
+  resp_data   <- getResponse(d)
+  m           <- makeCalibrationModel(d, dimensions, bound_cov)
+  item_models <- d@itemmap[, d@model_id]
+  item_models <- sanitizeItemModel(item_models, "mirt")
+  layout      <- mirt(resp_data, m, itemtype = item_models, pars = "values")
 
   return(layout)
 
 }
 
 #' @noRd
-filterItemParameters <- function(ipar) {
+sanitizeItemModel <- function(item_models, purpose) {
 
-  idx <- c()
+  if (purpose == "mirt") {
 
-  for (j in 1:dim(ipar)[2]) {
-    if (inherits(ipar[, j], "numeric")) {
-      if (any(ipar[, j] != round(ipar[, j]), na.rm = TRUE)) {
-        idx <- c(idx, j)
-      }
+    for (i in 1:length(item_models)) {
+      if (item_models[i] == "GPC") { item_models[i] <- "gpcm"  ; next }
+      if (item_models[i] == "GR")  { item_models[i] <- "graded"; next }
     }
+
   }
 
-  ipar <- ipar[, idx]
+  if (purpose == "plink") {
 
-  idx <- c(
-    grep("^a", names(ipar)),
-    grep("^b", names(ipar)),
-    grep("^c", names(ipar)),
-    grep("^d", names(ipar)),
-    grep("^a[1-9]", names(ipar)),
-    grep("^cb[1-9]", names(ipar))
-  )
+    for (i in 1:length(item_models)) {
+      if (item_models[i] == "GPC") { item_models[i] <- "gpcm"  ; next }
+      if (item_models[i] == "GR")  { item_models[i] <- "grm"; next }
+    }
 
-  ipar <- ipar[, unique(idx)]
+  }
 
-  return(ipar)
+  return(item_models)
 
 }
 
 #' @noRd
-extractAnchorParameters <- function(d, as_AD) {
+make_plink_poly_mod <- function(item_models) {
 
-  ipar <- filterItemParameters(d@anchor)
+  item_models <- sanitizeItemModel(item_models, "plink")
+
+  n_items <- length(item_models)
+  unique_item_models <- unique(item_models)
+
+  idx_items_by_model <- list()
+  for (item_model in unique_item_models) {
+    idx_items_by_model[[item_model]] <- which(item_models == item_model)
+  }
+
+  poly_mod <- plink::as.poly.mod(n_items, unique_item_models, idx_items_by_model)
+
+  return(poly_mod)
+
+}
+
+#' @noRd
+extractAnchorParameters <- function(d, as) {
+
+  ipar <- d@anchor
   rownames(ipar) <- d@anchor[, d@item_id]
 
-  if (as_AD) {
-    ipar              <- convertABtoAD(ipar)
-    anchor_dim        <- getAnchorDimension(d)
-    colnames(ipar)[1] <- sprintf("a%s", anchor_dim)
+  if (as == "AD") {
+    ipar <- convertABtoAD(ipar, d@item_id, d@model_id)
   }
 
   return(ipar)
@@ -166,59 +200,78 @@ extractAnchorParameters <- function(d, as_AD) {
 }
 
 #' @noRd
-convertABtoAD <- function(ipar) {
+extractLinkedParameters <- function(x, as, data) {
 
-  p_type <- detectParameterization(ipar)
+  if (inherits(x, "list")) {
 
-  if (p_type == "ab") {
+    # this is an output from plink::plink()
+    # this assumes AB format
 
-    dimensions <- detectDimensions(ipar)
-
-    ipar_a <- ipar[, 1:dimensions, drop = FALSE]
-    ipar_b <- ipar[, (dimensions + 1):dim(ipar)[2], drop = FALSE]
-
-    ipar_d <- ipar_b * NA
-    for (k in 1:dim(ipar_b)[2]) {
-      ipar_d[, k] <- -ipar_a * ipar_b[, k]
+    if (as == "AB") {
+      ipar <- x$pars@pars$From
+    }
+    if (as == "AD") {
+      ipar <- x$pars@pars$From
+      ipar <- convertABtoAD(ipar)
     }
-    colnames(ipar_d) <- sprintf("d%s", 1:dim(ipar_d)[2])
 
-    ipar_ad <- cbind(ipar_a, ipar_d)
+    # plink doesn't assign column names reliably for some reason
+    # fill it in
+    ipar <- data.frame(ipar)
+    if (as == "AB") {
+      colnames(ipar)[1] <- "a"
+      colnames(ipar)[1 + (1:(ncol(ipar) - 1))] <-
+        sprintf("b%s", 1:(ncol(ipar) - 1))
+    }
+    if (as == "AD") {
+      colnames(ipar)[1] <- "a"
+      colnames(ipar)[1 + (1:(ncol(ipar) - 1))] <-
+        sprintf("d%s", 1:(ncol(ipar) - 1))
+    }
 
-    return(ipar_ad)
+    item_ids    <- data.frame(data@itemmap[, data@item_id])
+    item_models <- data.frame(data@itemmap[, data@model_id])
+    names(item_ids)    <- data@item_id
+    names(item_models) <- data@model_id
+
+    ipar <- cbind(
+      item_ids,
+      item_models,
+      ipar
+    )
+
+    return(ipar)
 
   }
 
-  stop("unrecognized parameterization: cannot find cb* or b* columns")
+  if (inherits(x, "SingleGroupClass")) {
 
-}
+    # this is an output from mirt::mirt()
 
-#' @noRd
-convertADtoAB <- function(ipar) {
-
-  p_type <- detectParameterization(ipar)
-
-  if (p_type == "ad") {
-
-    dimensions <- detectDimensions(ipar)
-
-    ipar_a <- ipar[, 1:dimensions, drop = FALSE]
-    ipar_d <- ipar[, (dimensions + 1):dim(ipar)[2], drop = FALSE]
-
-    ipar_b <- ipar_d * NA
-    for (k in 1:dim(ipar_d)[2]) {
-      ipar_b[, k] <- -ipar_d[, k] / ipar_a
+    if (as == "AB") {
+      ipar <- mirt::coef(x, IRTpars = TRUE, simplify = TRUE)
+      ipar <- ipar$items
+    }
+    if (as == "AD") {
+      ipar <- mirt::coef(x, IRTpars = FALSE, simplify = TRUE)
+      ipar <- ipar$items
     }
-    colnames(ipar_b) <- sprintf("b%s", 1:dim(ipar_b)[2])
 
-    ipar_ab <- cbind(ipar_a, ipar_b)
+    item_ids    <- data.frame(data@itemmap[, data@item_id])
+    item_models <- data.frame(data@itemmap[, data@model_id])
+    names(item_ids)    <- data@item_id
+    names(item_models) <- data@model_id
 
-    return(ipar_ab)
+    ipar <- cbind(
+      item_ids,
+      item_models,
+      ipar
+    )
+
+    return(ipar)
 
   }
 
-  stop("unrecognized parameterization: cannot find d* columns")
-
 }
 
 #' @noRd
@@ -254,7 +307,27 @@ applyConstraintsToLayout <- function(layout, d, verbose) {
     verbose
   )
 
-  ipar_anchor <- extractAnchorParameters(d, as_AD = TRUE)
+  # this should be always in AD format because mirt uses AD
+  ipar_anchor <- extractAnchorParameters(d, as = "AD")
+  # also only keep the item parameters
+  keep_these <- unique(c(
+    grep("^a$"      , names(ipar_anchor), value = TRUE),
+    grep("^a[1-9]$" , names(ipar_anchor), value = TRUE),
+    grep("^b$"      , names(ipar_anchor), value = TRUE),
+    grep("^b[1-9]$" , names(ipar_anchor), value = TRUE),
+    grep("^cb$"     , names(ipar_anchor), value = TRUE),
+    grep("^cb[1-9]$", names(ipar_anchor), value = TRUE),
+    grep("^d$"      , names(ipar_anchor), value = TRUE),
+    grep("^d[1-9]$" , names(ipar_anchor), value = TRUE)
+  ))
+  ipar_anchor <- ipar_anchor[keep_these]
+
+  # mark anchor dimension
+  replace_this <- unique(c(
+    grep("^a$"      , names(ipar_anchor)),
+    grep("^a[1-9]$" , names(ipar_anchor))
+  ))
+  names(ipar_anchor)[replace_this] <- sprintf("a%s", anchor_dim)
 
   printLog(
     "constraints",
@@ -266,19 +339,20 @@ applyConstraintsToLayout <- function(layout, d, verbose) {
     verbose
   )
 
-  if (dimensions == 1 & (!"a1" %in% names(ipar_anchor))) {
+  if (dimensions == 1 && (!"a1" %in% names(ipar_anchor))) {
     # if using a 1D model and the anchor dimension is not 1
     a_par_name <- sprintf("a%s", getAnchorDimension(d))
     a_par_idx  <- which(names(ipar_anchor) == a_par_name)
     names(ipar_anchor)[a_par_idx] <- "a1"
   }
+
   par_to_fix     <- which(layout$item %in% rownames(ipar_anchor))
   n_items_to_fix <- length(unique(layout$item[par_to_fix]))
   layout$est[par_to_fix] <- FALSE
 
   for (i in 1:dim(ipar_anchor)[1]) {
     item_name <- rownames(ipar_anchor)[i]
-    for (j in 1:dim(ipar_anchor)[2]) {
+    for (j in 1:sum(!is.na(ipar_anchor[i, ]))) {
       par_name <- colnames(ipar_anchor)[j]
       idx <-
         layout$item == item_name &
@@ -349,6 +423,7 @@ applyConstraintsToLayout <- function(layout, d, verbose) {
 #' \code{\link{getRSSS}} is a function for generating a raw-score to standard-score crosswalk table.
 #'
 #' @param ipar an item parameter matrix for graded response items. Accepts both a/b and a/d format parameters. Accepts multidimensional item parameters.
+#' @param model_id the column name for item models.
 #' @param theta_grid the theta grid to use for numerical integration.
 #' @param is_minscore_0 if \code{TRUE}, the score of each item begins from 0.
 #' if \code{FALSE}, the score of each item begins from 1.
@@ -367,17 +442,20 @@ applyConstraintsToLayout <- function(layout, d, verbose) {
 #' o <- runCalibration(data_asq, technical = list(NCYCLES = 1000))
 #'
 #' ipar <- mirt::coef(o, IRTpars = TRUE, simplify = TRUE)$items
+#' ipar <- as.data.frame(ipar)
+#' ipar[, data_asq@model_id] <- data_asq@itemmap[, data_asq@model_id]
 #' items <- getItemNames(data_asq, 2)
 #'
 #' getRSSS(
 #'   ipar = ipar[items, ],
+#'   model_id = data_asq@model_id,
 #'   theta_grid = seq(-4, 4, .1),
 #'   is_minscore_0 = TRUE,
 #'   prior_mu_sigma = list(mu = 0, sigma = 1)
 #' )
 #' }
 #' @export
-getRSSS <- function(ipar, theta_grid, is_minscore_0, prior_mu_sigma) {
+getRSSS <- function(ipar, model_id, theta_grid, is_minscore_0, prior_mu_sigma) {
 
   if (is.vector(theta_grid)) {
     theta_grid <- matrix(theta_grid)
@@ -390,7 +468,7 @@ getRSSS <- function(ipar, theta_grid, is_minscore_0, prior_mu_sigma) {
   dimensions <- detectDimensions(ipar)
   n_cats <- detectNCategories(ipar)
 
-  pp <- computeResponseProbability(ipar, "grm", theta_grid)
+  pp <- computeResponseProbability(ipar, model_id, theta_grid)
   L  <- LWrecursion(pp, n_cats, theta_grid, is_minscore_0)
   o  <- LtoEAP(L, theta_grid, prior_mu_sigma)
 
@@ -427,14 +505,14 @@ getRSSS <- function(ipar, theta_grid, is_minscore_0, prior_mu_sigma) {
 }
 
 #' @noRd
-appendEscore <- function(score_table, n_scale, item_par_by_scale, min_score) {
+appendEscore <- function(score_table, n_scale, item_par_by_scale, model_id, min_score) {
 
   for (s in 1:(n_scale + 1)) {
     for (d in 1:(n_scale + 1)) {
       n_theta <- length(score_table[[s]]$eap)
       e_theta <- rep(NA, n_theta)
       for (i in 1:n_theta) {
-        e_theta[i] <- getEscoreTheta(item_par_by_scale[[d]], "grm", score_table[[s]]$eap[i], min_score[d] == 0)
+        e_theta[i] <- getEscoreTheta(item_par_by_scale[[d]], model_id, score_table[[s]]$eap[i], min_score[d] == 0)
       }
       e_name <- sprintf("escore_%s", names(score_table)[d])
       score_table[[s]][[e_name]] <- e_theta
@@ -501,16 +579,16 @@ detectParameterization <- function(ipar) {
 #'
 #' \code{\link{computeResponseProbability}} is an internal function for computing response probability from a set of item parameters.
 #'
-#' @param ipar a \code{\link{data.frame}} containing item parameters.
-#' @param model the item model to use. Accepts \code{grm} or {gpcm}.
+#' @param ipar a \code{\link{data.frame}} containing item model and item parameters.
+#' @param model_id the column name for item models.
 #' @param theta_grid theta values to compute probability values at.
 #'
 #' @return \code{\link{computeResponseProbability}} returns an item-wise list of probability matrices.
 #'
 #' @examples
-#' ipar <- PROsetta:::extractAnchorParameters(data_asq, FALSE)
+#' ipar <- PROsetta:::extractAnchorParameters(data_asq, as = "AB")
 #' theta_q <- seq(-4, 4, .1)
-#' p <- PROsetta:::computeResponseProbability(ipar, "grm", theta_q)
+#' p <- PROsetta:::computeResponseProbability(ipar, "item_model", theta_q)
 #'
 #' plot(
 #'   0, 0, type = "n", xlim = c(-4, 4), ylim = c(0, 1),
@@ -523,7 +601,7 @@ detectParameterization <- function(ipar) {
 #' lines(theta_q, p[[1]][, 5])
 #'
 #' @keywords internal
-computeResponseProbability <- function(ipar, model, theta_grid) {
+computeResponseProbability <- function(ipar, model_id, theta_grid) {
 
   if (is.vector(theta_grid)) {
     theta_grid <- matrix(theta_grid)
@@ -543,108 +621,112 @@ computeResponseProbability <- function(ipar, model, theta_grid) {
 
     # a/b parameterization
     if (p_type == "ab") {
-      par_a <- ipar[, 1:dimensions]
-      par_b <- ipar[, dimensions + 1:(max_cats - 1), drop = FALSE]
-      par_b <- as.matrix(par_b)
+      a_columns <- unique(c(
+        grep("^a$"     , names(ipar), value = TRUE),
+        grep("^a[1-9]$", names(ipar), value = TRUE)
+      ))
+      b_columns <- unique(c(
+        grep("^b$"      , names(ipar), value = TRUE),
+        grep("^b[1-9]$" , names(ipar), value = TRUE),
+        grep("^cb$"     , names(ipar), value = TRUE),
+        grep("^cb[1-9]$", names(ipar), value = TRUE)
+      ))
+      par_a <- ipar[, a_columns, drop = FALSE]
+      par_b <- ipar[, b_columns, drop = FALSE]
     }
 
+    # convert to matrix because TestDesign's cpp functions need it
+    par_a <- as.matrix(par_a)
+    par_b <- as.matrix(par_b)
+
     pp <- list()
     for (i in 1:ni) {
       pp[[i]] <- matrix(NA, nq, n_cats[i])
     }
 
-    if (model == "grm") {
-
-      for (i in 1:ni) {
+    for (i in 1:ni) {
 
+      if (ipar[i, model_id] == "GR") {
         pp[[i]] <- array_p_gr(
           theta_grid,
           par_a[i],
           par_b[i, 1:(n_cats[i] - 1)]
         )
-
+        next
       }
 
-      return(pp)
-    }
-
-    if (model == "gpcm") {
-
-      for (i in 1:ni) {
-
+      if (ipar[i, model_id] == "GPC") {
         pp[[i]] <- array_p_gpc(
           theta_grid,
           par_a[i],
           par_b[i, 1:(n_cats[i] - 1)]
         )
-
+        next
       }
 
-      return(pp)
-
     }
 
+    return(pp)
+
   }
 
   if (dimensions == 2) {
 
     # a/d parameterization
+    if (p_type == "ad") {
+      a_columns <- unique(c(
+        grep("^a$"     , names(ipar), value = TRUE),
+        grep("^a[1-9]$", names(ipar), value = TRUE)
+      ))
+      d_columns <- unique(c(
+        grep("^d$"      , names(ipar), value = TRUE),
+        grep("^d[1-9]$" , names(ipar), value = TRUE)
+      ))
+      par_a <- ipar[, a_columns, drop = FALSE]
+      par_d <- ipar[, d_columns, drop = FALSE]
+    }
 
-    par_a <- ipar[, 1:dimensions, drop = FALSE]
-    par_d <- ipar[, dimensions + 1:(max_cats - 1), drop = FALSE]
+    # convert to matrix because TestDesign's cpp functions need it
     par_a <- as.matrix(par_a)
     par_d <- as.matrix(par_d)
 
-    n_cats <- apply(par_d, 1, function(x) {
-      sum(!is.na(x)) + 1
-    })
-    max_cats <- max(n_cats)
-
     pp <- list()
     for (i in 1:ni) {
       pp[[i]] <- matrix(NA, nq, max_cats)
     }
 
-    if (model == "grm") {
-
-      for (i in 1:ni) {
+    for (i in 1:ni) {
 
+      if (ipar[i, model_id] == "GR") {
         pp[[i]] <- array_p_m_gr(
           theta_grid,
           par_a[i, , drop = FALSE],
-          par_d[i, 1:(n_cats[i] - 1), drop = FALSE]
+          par_d[i, , drop = FALSE]
         )
-
+        next
       }
 
-      return(pp)
-
-    }
-
-    if (model == "gpcm") {
-
-      for (i in 1:ni) {
-
+      if (ipar[i, model_id] == "GPC") {
         pp[[i]] <- array_p_m_gpc(
           theta_grid,
           par_a[i, , drop = FALSE],
-          par_d[i, 1:(n_cats[i] - 1), drop = FALSE]
+          par_d[i, , drop = FALSE]
         )
-
+        next
       }
 
     }
 
-  }
+    return(pp)
 
-  stop(sprintf("argument 'model': unrecognized value '%s'", model))
+  }
 
 }
 
 #' @noRd
-getEscoreTheta <- function(ipar, model, theta, is_minscore_0) {
+getEscoreTheta <- function(ipar, model_id, theta, is_minscore_0) {
 
-  pp <- computeResponseProbability(ipar, "grm", theta)
+  pp <- computeResponseProbability(ipar, model_id, theta)
 
   e  <- lapply(pp, function(x) {
     sum(x * 0:(length(x) - 1))
@@ -722,8 +804,8 @@ extractMuSigma <- function(calib) {
 }
 
 #' @noRd
-detectDimensions <- function(item_par_matrix) {
-  if ("a2" %in% colnames(item_par_matrix)) {
+detectDimensions <- function(ipar) {
+  if ("a2" %in% colnames(ipar)) {
     return(2)
   } else {
     return(1)
