@@ -20,7 +20,7 @@ NULL
 #' @param verbose if \code{TRUE}, print status messages. (default = \code{FALSE})
 #' @param ... additional arguments to pass onto \code{\link[mirt]{mirt}} in \href{https://CRAN.R-project.org/package=mirt}{'mirt'} package.
 #'
-#' @return \code{\link{runCalibration}} returns a \code{\linkS4class{SingleGroupClass}} object containing item calibration results.
+#' @return \code{\link{runCalibration}} returns a \code{\link[mirt:SingleGroupClass-class]{SingleGroupClass}} object containing item calibration results.
 #'
 #' This object can be used in \code{\link[mirt:coef-method]{coef}}, \code{\link[mirt]{itemfit}}, \code{\link[mirt]{itemplot}} in \href{https://CRAN.R-project.org/package=mirt}{'mirt'} package to extract wanted information.
 #'
@@ -76,13 +76,18 @@ runCalibration <- function(
     )
 
     bound_cov   <- FALSE
-    layout  <- makeParameterLayout(data, dimensions, bound_cov)
-    layout  <- applyConstraintsToLayout(layout, data, verbose)
+    layout      <- makeParameterLayout(data, dimensions, bound_cov)
+    layout      <- applyConstraintsToLayout(layout, data, verbose)
     model_specs <- makeCalibrationModel(data, dimensions, bound_cov)
-    calibration <- mirt::mirt(resp_data, model_specs, itemtype = "graded", pars = layout, ...)
+    item_models <- data@itemmap[, data@model_id]
+    item_models <- sanitizeItemModel(item_models, "mirt")
+    calibration <- mirt::mirt(resp_data, model_specs, itemtype = item_models, pars = layout, ...)
 
   } else if (toupper(fix_method) == "THETA") {
 
+    # used for calibrated projection
+    # not used in 1D cases
+
     anchor_dim <- getAnchorDimension(data)
     printLog(
       "CPFIXEDDIM",
@@ -165,7 +170,10 @@ runCalibration <- function(
     bound_cov   <- TRUE
     layout      <- makeParameterLayout(data, dimensions, bound_cov)
     model_specs <- makeCalibrationModel(data, dimensions, bound_cov)
-    calibration <- mirt::mirt(resp_data, model_specs, itemtype = "graded", pars = layout, ...)
+    item_models <- data@itemmap[, data@model_id]
+    item_models <- sanitizeItemModel(item_models, "mirt")
+    calibration <- mirt::mirt(resp_data, model_specs, itemtype = item_models, pars = layout, ...)
+
   }
 
   if (calibration@OptimInfo$iter == calibration@Options$NCYCLES) {
@@ -252,13 +260,13 @@ runLinking <- function(data, method, verbose = FALSE, ...) {
     ipar      <- fit$items
     ni_all    <- nrow(ipar)
     ni_anchor <- nrow(data@anchor)
-    ipar_anchor     <- extractAnchorParameters(data, FALSE)
+    ipar_anchor     <- extractAnchorParameters(data, as = "AB")
     n_cats_anchor   <- detectNCategories(ipar_anchor)
     max_cats_anchor <- max(n_cats_anchor)
     id_new <- data.frame(New = 1:ni_all   , ID = data@itemmap[[data@item_id]])
     id_old <- data.frame(Old = 1:ni_anchor, ID = data@anchor[[data@item_id]])
     common <- merge(id_new, id_old, by = "ID", sort = FALSE)[c("New", "Old")]
-    
+
     if (method %in% c("MM", "MS", "HB", "SL")) {
 
       printLog(
@@ -272,8 +280,8 @@ runLinking <- function(data, method, verbose = FALSE, ...) {
         verbose
       )
 
-      pm_all    <- plink::as.poly.mod(ni_all   , "grm", 1:ni_all)
-      pm_anchor <- plink::as.poly.mod(ni_anchor, "grm", 1:ni_anchor)
+      pm_all    <- make_plink_poly_mod(data@itemmap[, data@model_id])
+      pm_anchor <- make_plink_poly_mod(data@anchor[, data@model_id])
       n_cats <- list(
         getColumn(data@itemmap, "ncat"),
         n_cats_anchor
@@ -289,45 +297,39 @@ runLinking <- function(data, method, verbose = FALSE, ...) {
         grp.names = c("From", "To")
       )
 
-      out <- plink::plink(plink_pars, rescale = method, base.grp = 2)
-      out$constants <- out$link@constants[[method]]
-      out$ipar_linked <- out$pars@pars$From
-      out$ipar_anchor <- out$pars@pars$To
+      o <- plink::plink(plink_pars, rescale = method, base.grp = 2)
+      o$constants <- o$link@constants[[method]]
+      o$ipar_linked <- extractLinkedParameters(o, as = "AB", data)
+      o$ipar_anchor <- extractAnchorParameters(data, as = "AB")
 
     }
 
     if (method == "FIXEDPAR") {
 
-      out <- list()
-      out$constants <- NA
-      out$ipar_linked <- ipar
-      out$ipar_anchor <- extractAnchorParameters(data, as_AD = FALSE)
-      out$mu_sigma    <- extractMuSigma(calibration)
+      o <- list()
+      o$constants <- NA
+      o$ipar_linked <- extractLinkedParameters(calibration, as = "AB", data)
+      o$ipar_anchor <- extractAnchorParameters(data, as = "AB")
+      o$mu_sigma    <- extractMuSigma(calibration)
 
     }
 
-    out$method      <- method
-    rownames(out$ipar_linked) <- id_new$ID
-    rownames(out$ipar_anchor) <- id_old$ID
-    colnames(out$ipar_linked) <- colnames(ipar)[1:dim(out$ipar_linked)[2]]
-    colnames(out$ipar_anchor) <- colnames(ipar)[1:dim(out$ipar_anchor)[2]]
+    o$method <- method
 
-    return(out)
+    return(o)
 
   }
 
   if (dimensions == 2) {
 
-    pars <- mirt::coef(calibration, IRTpars = FALSE, simplify = TRUE)
+    o <- list()
+    o$constants   <- NA
+    o$ipar_linked <- extractLinkedParameters(calibration, as = "AD", data)
+    o$ipar_anchor <- extractAnchorParameters(data, as = "AD")
+    o$mu_sigma    <- extractMuSigma(calibration)
+    o$method      <- method
 
-    out <- list()
-    out$constants   <- NA
-    out$ipar_linked <- pars$items
-    out$ipar_anchor <- extractAnchorParameters(data, as_AD = TRUE)
-    out$mu_sigma    <- extractMuSigma(calibration)
-    out$method      <- method
-
-    return(out)
+    return(o)
 
   }
 
@@ -351,7 +353,7 @@ runLinking <- function(data, method, verbose = FALSE, ...) {
 #' @param verbose if \code{TRUE}, print status messages. (default = \code{FALSE})
 #' @param ... other arguments to pass onto \code{\link[equate]{equate}}.
 #'
-#' @return \code{\link{runEquateObserved}} returns an \code{\link{equate}} object containing the test equating result.
+#' @return \code{\link{runEquateObserved}} returns an \code{\link[equate]{equate}} object containing the test equating result.
 #'
 #' The printed summary statistics indicate the distributional properties of the two supplied scales and the equated scale.
 #' \itemize{
@@ -569,20 +571,20 @@ runRSSS <- function(
   # the last item_par_by_scale is the combined scale
 
   if (n_scale == 1) {
-    score_table <- getRSSS(item_par_by_scale[[n_scale + 1]], theta_grid, min_score == 0, prior_mu_sigma)
+    score_table <- getRSSS(item_par_by_scale[[n_scale + 1]], data@model_id, theta_grid, min_score == 0, prior_mu_sigma)
     return(score_table)
   } else if (n_scale > 1) {
     score_table <- vector(mode = "list", length = n_scale + 1)
 
     for (s in 1:(n_scale + 1)) {
-      score_table[[s]] <- getRSSS(item_par_by_scale[[s]], theta_grid, min_score[s] == 0, prior_mu_sigma)
+      score_table[[s]] <- getRSSS(item_par_by_scale[[s]], data@model_id, theta_grid, min_score[s] == 0, prior_mu_sigma)
       colnames(score_table[[s]])[1] <- sprintf("raw_%i", s)
     }
 
     names(score_table) <- names(item_par_by_scale)
 
     if (dimensions == 1 & link_method != "CPLA") {
-      score_table <- appendEscore(score_table, n_scale, item_par_by_scale, min_score)
+      score_table <- appendEscore(score_table, n_scale, item_par_by_scale, data@model_id, min_score)
     }
     if (dimensions == 1 & link_method == "CPLA") {
       score_table <- appendCPLA(score_table, n_scale, ipar_linked$mu_sigma)
