@@ -9,7 +9,6 @@ NULL
 #' @param data a \code{\linkS4class{PROsetta_data}} object.
 #' @param ipar a \code{\link{data.frame}} containing item parameters.
 #' @param scale the index of the scale to use. \code{combined} refers to the combined scale. (default = \code{combined})
-#' @param model the item model to use. Accepts \code{grm} or \code{gpcm}. (default = \code{grm})
 #' @param theta_grid the theta grid to use for numerical integration. (default = \code{seq(-4, 4, .1)})
 #' @param prior_dist the type of prior distribution. Accepts \code{normal} or \code{logistic}. (default = \code{normal})
 #' @param prior_mean mean of the prior distribution. (default = \code{0.0})
@@ -34,7 +33,7 @@ NULL
 #'
 #' @export
 getTheta <- function(
-  data, ipar, scale = "combined", model = "grm",
+  data, ipar, scale = "combined",
   theta_grid = seq(-4, 4, .1),
   prior_dist = "normal",
   prior_mean = 0.0,
@@ -64,16 +63,16 @@ getTheta <- function(
   prior_mu_sigma$sigma <- prior_sd ** 2
 
   prior <- generatePriorDensity(theta_grid, prior_dist, prior_mu_sigma)
-  pp    <- computeResponseProbability(ipar, model, theta_grid)
+  pp    <- computeResponseProbability(ipar, data@model_id, theta_grid)
   eap   <- getEAP(theta_grid, prior, pp, resp_data)
   eap   <- cbind(person_id, eap)
 
-	out <- list()
-	out$theta    <- eap
-	out$item_idx <- item_idx
-	out$ipar     <- ipar
+	o <- list()
+	o$theta    <- eap
+	o$item_idx <- item_idx
+	o$ipar     <- ipar
 
-	return(out)
+	return(o)
 
 }
 
@@ -141,23 +140,23 @@ compareScores <- function(
   left, right, type = c("corr", "mean", "sd", "rmsd", "mad")
 ) {
 
-  out <- list()
+  o <- list()
   if ("corr" %in% type) {
-    out$corr <- cor(left, right)
+    o$corr <- cor(left, right)
   }
   if ("mean" %in% type) {
-    out$mean <- mean(left - right)
+    o$mean <- mean(left - right)
   }
   if ("sd" %in% type) {
-    out$sd <- sqrt(mean((left - right)**2) - mean(left - right)**2)
+    o$sd <- sqrt(mean((left - right)**2) - mean(left - right)**2)
   }
   if ("rmsd" %in% type) {
-    out$rmsd <- sqrt(mean((left - right)**2))
+    o$rmsd <- sqrt(mean((left - right)**2))
   }
   if ("mad" %in% type) {
-    out$mad <- mean(abs(left - right))
+    o$mad <- mean(abs(left - right))
   }
 
-  return(as.data.frame(out))
+  return(as.data.frame(o))
 
 }
